@@ -1,5 +1,5 @@
 /* gifwrite.c - Functions to write GIFs.
-   Copyright (C) 1997-9 Eddie Kohler, eddietwo@lcs.mit.edu
+   Copyright (C) 1997-2000 Eddie Kohler, eddietwo@lcs.mit.edu
    This file is part of the GIF library.
 
    The GIF library is free software*. It is distributed under the GNU Public
@@ -74,13 +74,13 @@ typedef struct Gif_Context {
 } Gif_Context;
 
 
-
 typedef struct Gif_Writer {
   
   FILE *f;
   byte *v;
   u_int32_t pos;
   u_int32_t cap;
+  int flags;
   void (*byte_putter)(byte, struct Gif_Writer *);
   void (*block_putter)(byte *, u_int16_t, struct Gif_Writer *);
   
@@ -257,8 +257,9 @@ write_compressed_data(byte **img, u_int16_t width, u_int16_t height,
        time around. */
     while (height != 0) {
       suffix = *imageline;
-      /* if (suffix >= clear_code) suffix = 0; -- can never happen, because
-	 of how we determine min_code_bits in write_image_data */
+      if (suffix >= clear_code)
+	/* should not happen unless GIF_WRITE_CAREFUL_MIN_CODE_BITS */
+	suffix = 0;
       if (!work_node)
 	next_node = &gfc->nodes[suffix];
       else if (work_node->type == TABLE_TYPE)
@@ -331,30 +332,64 @@ write_compressed_data(byte **img, u_int16_t width, u_int16_t height,
 
 
 static int
-write_image_data(byte **img, u_int16_t width, u_int16_t height,
-		 byte interlaced, Gif_Context *gfc, Gif_Writer *grr)
+calculate_min_code_bits(Gif_Stream *gfs, Gif_Image *gfi, Gif_Writer *grr)
 {
-  int i;
-  u_int16_t x, y, max_color;
-  byte min_code_bits;
+  int colors_used = -1, min_code_bits, i;
+
+  if (grr->flags & GIF_WRITE_CAREFUL_MIN_CODE_SIZE) {
+    /* calculate m_c_b based on colormap */
+    if (gfi->local && gfi->local->ncol > 0)
+      colors_used = gfi->local->ncol;
+    else if (gfs->global && gfs->global->ncol > 0)
+      colors_used = gfs->global->ncol;
+    
+  } else if (gfi->compressed) {
+    /* take m_c_b from compressed image */
+    colors_used = 1 << gfi->compressed[0];
   
-  max_color = 0;
-  for (y = 0; y < height && max_color < 128; y++) {
-    byte *data = img[y];
-    for (x = width; x > 0; x--, data++)
-      if (*data > max_color)
-	max_color = *data;
+  } else if (gfi->img) {
+    /* calculate m_c_b from uncompressed data */
+    int x, y, width = gfi->width, height = gfi->height;
+    colors_used = 0;
+    for (y = 0; y < height && colors_used < 128; y++) {
+      byte *data = gfi->img[y];
+      for (x = width; x > 0; x--, data++)
+	if (*data > colors_used)
+	  colors_used = *data;
+    }
+    colors_used++;
+    
+  } else {
+    /* should never happen */
+    colors_used = 256;
   }
   
   min_code_bits = 2;		/* min_code_bits of 1 isn't allowed */
   i = 4;
-  while (i < max_color + 1) {
+  while (i < colors_used) {
     min_code_bits++;
     i *= 2;
   }
+
+  if ((grr->flags & GIF_WRITE_CAREFUL_MIN_CODE_SIZE)
+      && gfi->compressed && gfi->compressed[0] != min_code_bits) {
+    /* if compressed image disagrees with careful min_code_bits, recompress */
+    if (Gif_UncompressImage(gfi))
+      Gif_FullCompressImage(gfs, gfi, grr->flags);
+  }
   
-  if (interlaced) {
-    int y;
+  return min_code_bits;
+}
+
+static int
+write_image_data(Gif_Image *gfi, byte min_code_bits,
+		 Gif_Context *gfc, Gif_Writer *grr)
+{
+  byte **img = gfi->img;
+  u_int16_t width = gfi->width, height = gfi->height;
+    
+  if (gfi->interlace) {
+    u_int16_t y;
     byte **nimg = Gif_NewArray(byte *, height + 1);
     if (!nimg) return 0;
     
@@ -373,14 +408,17 @@ write_image_data(byte **img, u_int16_t width, u_int16_t height,
 
 
 int
-Gif_CompressImage(Gif_Stream *gfs, Gif_Image *gfi)
+Gif_FullCompressImage(Gif_Stream *gfs, Gif_Image *gfi, int flags)
 {
   int ok = 0;
+  byte min_code_bits;
   Gif_Writer grr;
   Gif_Context gfc;
   
-  if (gfi->compressed && gfi->free_compressed)
+  if (gfi->compressed && gfi->free_compressed) {
     (*gfi->free_compressed)((void *)gfi->compressed);
+    gfi->compressed = 0;
+  }
   
   gfc.nodes = Gif_NewArray(Gif_Node, NODES_SIZE);
   gfc.links = Gif_NewArray(Gif_Node *, LINKS_SIZE);
@@ -390,12 +428,13 @@ Gif_CompressImage(Gif_Stream *gfs, Gif_Image *gfi)
   grr.cap = 1024;
   grr.byte_putter = memory_byte_putter;
   grr.block_putter = memory_block_putter;
+  grr.flags = flags;
   
   if (!gfc.nodes || !gfc.links || !grr.v)
     goto done;
-  
-  ok = write_image_data(gfi->img, gfi->width, gfi->height, gfi->interlace,
-			&gfc, &grr);
+
+  min_code_bits = calculate_min_code_bits(gfs, gfi, &grr);
+  ok = write_image_data(gfi, min_code_bits, &gfc, &grr);
   
  done:
   if (!ok) {
@@ -441,7 +480,7 @@ write_color_table(Gif_Color *c, int ncol, Gif_Writer *grr)
 static int
 write_image(Gif_Stream *gfs, Gif_Image *gfi, Gif_Context *gfc, Gif_Writer *grr)
 {
-  byte packed = 0;
+  byte min_code_bits, packed = 0;
   
   gifputbyte(',', grr);
   gifputunsigned(gfi->left, grr);
@@ -461,6 +500,10 @@ write_image(Gif_Stream *gfs, Gif_Image *gfi, Gif_Context *gfc, Gif_Writer *grr)
   
   if (gfi->local && gfi->local->ncol > 0)
     write_color_table(gfi->local->col, gfi->local->ncol, grr);
+
+  /* calculate min_code_bits here (because calculation may involve
+     recompression, if GIF_WRITE_CAREFUL_MIN_CODE_BITS is true) */
+  min_code_bits = calculate_min_code_bits(gfs, gfi, grr);
   
   /* use existing compressed data if it exists. This will tend to whip
      people's asses who uncompress an image, keep the compressed data around,
@@ -476,8 +519,7 @@ write_image(Gif_Stream *gfs, Gif_Image *gfi, Gif_Context *gfc, Gif_Writer *grr)
     }
     
   } else
-    write_image_data(gfi->img, gfi->width, gfi->height, gfi->interlace,
-		     gfc, grr);
+    write_image_data(gfi, min_code_bits, gfc, grr);
   
   return 1;
 }
@@ -674,16 +716,33 @@ write_gif(Gif_Stream *gfs, Gif_Writer *grr)
 
 
 int
-Gif_WriteFile(Gif_Stream *gfs, FILE *f)
+Gif_FullWriteFile(Gif_Stream *gfs, int flags, FILE *f)
 {
   Gif_Writer grr;
   grr.f = f;
   grr.byte_putter = file_byte_putter;
   grr.block_putter = file_block_putter;
+  grr.flags = flags;
   return write_gif(gfs, &grr);
 }
 
 
+#undef Gif_CompressImage
+#undef Gif_WriteFile
+
+int
+Gif_CompressImage(Gif_Stream *gfs, Gif_Image *gfi)
+{
+  return Gif_FullCompressImage(gfs, gfi, 0);
+}
+
+int
+Gif_WriteFile(Gif_Stream *gfs, FILE *f)
+{
+  return Gif_FullWriteFile(gfs, 0, f);
+}
+
+
 #ifdef __cplusplus
 }
 #endif
